import mnist
import mnist_network
import numpy as np

from network import Network, train

def test_load():
    """Tests whether the test data can be loaded."""
    print("Loading the test data")
    test_data = mnist_network.read_test_data()
    print("The test data has been loaded.")
    print("Number of instances: %d" % len(test_data))
    if len(test_data) == 10000:
        print("[OK] Test succeeded.")
    print()

def evaluate(network, test_data):
    """
    This computes the error rate of "network" when evaluated against the test
    data in "test_data". The test data is a list of pairs "(x, y)" where "x" is
    an image vector and "y" is the gold-standard digit (represented as an
    integer between 0 and 9). The error rate is defined as the fraction of
    incorrectly classified digits; note that this is a number between 0 and 1.
    """

    return 0

def test_evaluate():
    """Tests the evaluation function."""
    print("Loading the test data")
    test_data = mnist_network.read_test_data()

    print("Loading the pretrained network from net.pkl")
    net = Network.load("net.pkl")

    expected = 0.0437
    expected_precentage = "{:.2f}".format(expected*100)
    error_rate = evaluate(net, test_data)
    error_precentage = "{:.2f}".format(error_rate*100)

    if expected_precentage == error_precentage:
        print("[OK] Test succeeded.")
    else:
        print(f"[ERROR] Expected {expected} but got {error_rate}.")
    print()

def test_train():
    """Tests the training procedure."""
    
    print("Loading the training data")
    training_data = mnist_network.read_training_data()
    print("Loading the test data")
    test_data = mnist_network.read_test_data()
    
    # Run experiments

    # Experimentera med antalet noder i dolda lagret (default: 20)
    net = Network([784, 20, 10])
    # TODO: Experimentera med värdet på inlärningsparametern eta (default: 0.5)
    # TODO: VG: Experimentera med antal epoker (default: 1)
    
    train(network=net, data=training_data, n_epochs=1, batch_size=10,
        eta=0.5, evaluate=lambda n: evaluate(n, test_data))
    print()


if __name__ == "__main__":    
    # Kommentera ut denna när ni testat att det funkar
    test_load()

    # Kör denna rad för att testa evaluate()
    #test_evaluate()
    
    # Kör denna rad för att köra resten av experimenten
    #test_train()
