#!/usr/bin/env python

# A reader for the MNIST data format; see http://yann.lecun.com/exdb/mnist/
# Marco Kuhlmann <marco.kuhlmann@liu.se>

import gzip
import numpy as np
import struct

def read_images(input):
    magic = struct.unpack('>BBBB', input.read(4))
    assert magic[0] == 0 and magic[1] == 0 and magic[2] == 8 and magic[3] == 3
    n_imgs = struct.unpack('>i', input.read(4))[0]
    n_rows = struct.unpack('>i', input.read(4))[0]
    n_cols = struct.unpack('>i', input.read(4))[0]
    n_pxls = n_rows * n_cols
    images = []
    for i in range(n_imgs):
        images.append(struct.unpack('>%dB' % n_pxls, input.read(n_pxls)))
    return images

def read_labels(input):
    magic = struct.unpack('>BBBB', input.read(4))
    assert magic[0] == 0 and magic[1] == 0 and magic[2] == 8 and magic[3] == 1
    n_labs = struct.unpack('>i', input.read(4))[0]
    labels = []
    for i in range(n_labs):
        labels.append(struct.unpack('>B', input.read(1))[0])
    return labels

def read_data(images_file, labels_file):
    images = []
    labels = []
    with gzip.open(images_file) as f:
        images = read_images(f)
    with gzip.open(labels_file) as f:
        labels = read_labels(f)
    assert len(images) == len(labels)
    return list(zip(images, labels))

def read_training_data():
    return read_data('train-images-idx3-ubyte.gz', 'train-labels-idx1-ubyte.gz')

def read_test_data():
    return read_data('t10k-images-idx3-ubyte.gz', 't10k-labels-idx1-ubyte.gz')

if __name__ == "__main__":
    print("Test data: %d items" % len(read_test_data()))
